//
//  ColorDescriptor.cpp
//  tilemaker
//
//  Created by Attila Zsaki on 2015-10-28.
//  Copyright © 2015 Attila Zsaki. All rights reserved.
//

#include <cstdlib>
#include <iostream>
#include <math.h>

#include "ColorDescriptor.hpp"
#include "Noise.hpp"

using namespace std;

ColorDescriptor::ColorDescriptor()
{
    color.SetColor(1.0,1.0,1.0,1.0);
    colorLowSide.SetColor(1.0,1.0,1.0,1.0);;
    colorHighSide.SetColor(1.0,1.0,1.0,1.0);;
    
    colorType="solid"; // solid, variable
    colorNoiseType="random"; // random, noiseFunctionName(perlin,sineModulated,marble,wood),
    noiseXMin=0.0;
    noiseYMin=0.0;
    noiseXMax=1.0;
    noiseYMax=1.0;
    
    octaves=1;
    persistence=1.0;
    
    // cosine modulated function
    parameterA=1.0;
}

ColorDescriptor::~ColorDescriptor()
{
    
}

ColorRGBA ColorDescriptor::GetColor(double x, double y, double* noiseValue)
{
    ColorRGBA finalColor=ColorRGBA();
    
    if (colorType=="solid") {
        finalColor=color;
        *noiseValue=1.0;
    }
    if (colorType=="variable") {
        
        if (colorNoiseType=="random") {
            
            UtilityFunctions utility;
            
            double randomNumber=utility.RandomNumberInRange(0,1);
            
            *noiseValue=randomNumber;
            
            if (randomNumber<0.5) {
                finalColor=ColorLerp(colorLowSide,color,randomNumber);
            }
            else {
                finalColor=ColorLerp(color,colorHighSide,randomNumber);
            }
        }
        
        if (colorNoiseType=="perlin") {
            
            PerlinNoise noise=PerlinNoise();
                        
            double noiseX=noiseXMin+(noiseXMax-noiseXMin)*x;
            double noiseY=noiseYMin+(noiseYMax-noiseYMin)*y;
            
            double randomNumber=noise.OctavePerlin(noiseX,noiseY,0.0,octaves,persistence);
            
            *noiseValue=randomNumber;
            
            if (randomNumber<0.5) {
                finalColor=ColorLerp(colorLowSide,color,randomNumber);
            }
            else {
                finalColor=ColorLerp(color,colorHighSide,randomNumber);
            }
        }
        if (colorNoiseType=="cosineModulated") {
            
            PerlinNoise noise=PerlinNoise();
            
            double noiseX=noiseXMin+(noiseXMax-noiseXMin)*x;
            double noiseY=noiseYMin+(noiseYMax-noiseYMin)*y;
            
            double randomNumber=noise.OctavePerlin(noiseX,noiseY,0.0,octaves,persistence);
            
            randomNumber=0.5+cos(noiseX+randomNumber*parameterA)*0.5;
            
            *noiseValue=randomNumber;
            
            if (randomNumber<0.5) {
                finalColor=ColorLerp(colorLowSide,color,randomNumber);
            }
            else {
                finalColor=ColorLerp(color,colorHighSide,randomNumber);
            }
        }
        if (colorNoiseType=="wood") {
            
            PerlinNoise noise=PerlinNoise();
            
            double noiseX=noiseXMin+(noiseXMax-noiseXMin)*x;
            double noiseY=noiseYMin+(noiseYMax-noiseYMin)*y;
            
            double randomNumber=noise.OctavePerlin(noiseX,noiseY,0.0,octaves,persistence);
            
            randomNumber=randomNumber*parameterA-(int)(randomNumber*parameterA);
            
            *noiseValue=randomNumber;
            
            if (randomNumber<0.5) {
                finalColor=ColorLerp(colorLowSide,color,randomNumber);
            }
            else {
                finalColor=ColorLerp(color,colorHighSide,randomNumber);
            }
        }

    }
    
    
    return (finalColor);
}

ColorRGBA ColorDescriptor::ColorLerp(ColorRGBA a, ColorRGBA b, double x)
{
    ColorRGBA color;
    
    color.dR=Lerp(a.dR,b.dR,x);
    color.dG=Lerp(a.dG,b.dG,x);
    color.dB=Lerp(a.dB,b.dB,x);
    color.dA=Lerp(a.dA,b.dA,x);
    
    return (color);
}

