//
//  PerlinNoise.hpp
//  tilemaker
//
//  Created by Attila Zsaki on 2015-10-27.
//  Copyright © 2015 Attila Zsaki. All rights reserved.
//
//  NOTE: This source code was heavily inspired by the one discussed at:
//        http://flafla2.github.io/2014/08/09/perlinnoise.html
//
//        THus I cannot claim that I wrote it. All credits go to the original author!

#ifndef PerlinNoise_hpp
#define PerlinNoise_hpp

#include <stdio.h>

class PerlinNoise
{
public:
    PerlinNoise();
    ~PerlinNoise();
    
    void SetRepeat(int repeat);
    
    
    double OctavePerlin(double x, double y, double z, int octaves, double persistence);
    double Perlin(double x, double y, double z); 
    
private:
    
    int Inc(int num);
    double Fade(double t);
    double Lerp(double a, double b, double x);
    double Grad(int hash, double x, double y, double z);
    
private:
    
    int iRepeat;
        
    static const int permutation[256];
    int perm2[512];
    
    
    double PI;
};

inline void PerlinNoise::SetRepeat(int repeat)
{
    iRepeat=repeat;
}

inline int PerlinNoise::Inc(int num)
{
    num++;
    if (iRepeat>0) {
        num%=iRepeat;
    }
    
    return (num);
}

inline double PerlinNoise::Fade(double t)
{
    // Fade function as defined by Ken Perlin.  This eases coordinate values
    // so that they will "ease" towards integral values.  This ends up smoothing
    // the final output.
    return (t*t*t*(t*(t*6-15)+10));			// 6t^5 - 15t^4 + 10t^3
}

inline double PerlinNoise::Lerp(double a, double b, double x)
{
    return (a+x*(b-a));
}

#endif /* Noise_hpp */
