//
//  RectangularTileMaker.hpp
//  tilemaker
//
//  Created by Attila Zsaki on 2015-10-28.
//  Copyright © 2015 Attila Zsaki. All rights reserved.
//

#ifndef RectangularTileMaker_hpp
#define RectangularTileMaker_hpp

#include <stdio.h>
#include <string>
#include <Magick++.h>

#include "ColorRGBA.hpp"
#include "ColorDescriptor.hpp"

using namespace std;
using namespace Magick;

class RectangularTileMaker
{
public:
    RectangularTileMaker();
    ~RectangularTileMaker();
    
    void SetTiledAreaWidth(double tiledAreaWidth);
    void SetTiledAreaLength(double tiledAreaLength);
    
    void SetTileWidth(double tileWidth);
    void SetTileLength(double tileLength);
    
    void SetGroutWidth(double groutWidth);
    
    void SetTileXOffset(double tileXOffset);
    void SetTileYOffset(double tileYOffset);
    
    ColorDescriptor* GetTileColorDescriptorGlobal();
    ColorDescriptor* GetTileColorDescriptorLocal();
    ColorDescriptor* GetGroutColorDescriptor();
    ColorDescriptor* GetTileBumpNoiseAlternate();
    ColorDescriptor* GetTileDiscolorationMap();
    
    void SetTextureWidth(int textureWidth);
    void SetTextureHeight(int textureHeight);
    
    void GenerateTexture();
    
    void SetRandomTileOrientation(bool flag);
    void SetRandomTileNoiseStartPoint(bool flag);
    
    void UseTileXOffset();
    
    void SetGenerateBumpMap(bool flag);

    void SetMaxGroutBumpHeight(double value);
    void SetMaxTileBumpHeight(double value);
    

    void SetTileBumpNoiseAlternate(bool flag);
    
    void GenerateTileDiscolorationMap(bool flag);
    
    void DiscolorationMapTheresholdLower(double value);
    void DiscolorationMapTheresholdUpper(double value);
    void DiscolorationMapTheresholdPercentReduction(double value);
    
    void BlurFinalImage(bool flag);
    
    void SetOutputFileName(string fileName);
    
private:
    
    void TileToTextureSpace(double x, double y, int* i, int* j);
    void TextureToTileSpace(int i, int j, double* x, double* y);
    
    void RenderGrout(Image* textureImage,Image* bumpImage);
    void RenderTiles(Image* textureImage,Image* bumpImage);
    void RenderDiscoloration(Image* textureImage);
 
private:
    
    double dTiledAreaWidth;
    double dTiledAreaLength;
    
    double dTileWidth;
    double dTileLength;
    
    double dGroutWidth;
    
    double dTileXOffset;

    
    ColorDescriptor tileColorDescriptorGlobal;
    ColorDescriptor tileColorDescriptorLocal;
    ColorDescriptor groutColorDescriptor;
    
    int iTextureWidth;
    int iTextureHeight;
    
    bool bRandomTileOrientation;
    bool bRandomTileNoiseStartPoint;
    
    bool bUseTileXOffset;
    
    bool bGenerateBumpMap;

    double dMaxGroutBumpHeight;
    double dMaxTileBumpHeight;
    
    bool bTileBumpNoiseAlternate;
    
    ColorDescriptor tileColorDescriptorBump;
    
    bool bGenerateDiscolorationMap;
    ColorDescriptor tileColorDescriptorDiscoloration;
    double dDiscolorationThresholdLower;
    double dDiscolorationThresholdUpper;
    double dDiscolorationThresholdPercentReduction;
    
    bool bBlurFinalImage;
    
    string outputFileName;
    
};

inline void RectangularTileMaker::SetTiledAreaWidth(double tiledAreaWidth)
{
    dTiledAreaWidth=tiledAreaWidth;
}

inline void RectangularTileMaker::SetTiledAreaLength(double tiledAreaLength)
{
    dTiledAreaLength=tiledAreaLength;
}

inline void RectangularTileMaker::SetTileWidth(double tileWidth)
{
    dTileWidth=tileWidth;
}

inline void RectangularTileMaker::SetTileLength(double tileLength)
{
    dTileLength=tileLength;
}

inline void RectangularTileMaker::SetGroutWidth(double groutWidth)
{
    dGroutWidth=groutWidth;
}

inline void RectangularTileMaker::SetTileXOffset(double tileXOffset)
{
    dTileXOffset=tileXOffset;
}

inline ColorDescriptor* RectangularTileMaker::GetTileColorDescriptorGlobal()
{
    return (&tileColorDescriptorGlobal);
}

inline ColorDescriptor* RectangularTileMaker::GetTileColorDescriptorLocal()
{
    return (&tileColorDescriptorLocal);
}

inline ColorDescriptor* RectangularTileMaker::GetGroutColorDescriptor()
{
    return (&groutColorDescriptor);
}

inline void RectangularTileMaker::SetTextureWidth(int textureWidth)
{
    iTextureWidth=textureWidth;
}

inline void RectangularTileMaker::SetTextureHeight(int textureHeight)
{
    iTextureHeight=textureHeight;
}

inline void RectangularTileMaker::SetRandomTileOrientation(bool flag)
{
    bRandomTileOrientation=flag;
}

inline void RectangularTileMaker::SetRandomTileNoiseStartPoint(bool flag)
{
    bRandomTileNoiseStartPoint=flag;
}

inline void RectangularTileMaker::UseTileXOffset()
{
    bUseTileXOffset=true;
}

inline void RectangularTileMaker::SetMaxGroutBumpHeight(double value)
{
    dMaxGroutBumpHeight=value;
}

inline void RectangularTileMaker::SetMaxTileBumpHeight(double value)
{
    dMaxTileBumpHeight=value;
}

inline void RectangularTileMaker::SetGenerateBumpMap(bool flag)
{
    bGenerateBumpMap=flag;
}

inline void RectangularTileMaker::SetTileBumpNoiseAlternate(bool flag)
{
    bTileBumpNoiseAlternate=flag;
}

inline ColorDescriptor* RectangularTileMaker::GetTileBumpNoiseAlternate()
{
    return (&tileColorDescriptorBump);
}

inline void RectangularTileMaker::GenerateTileDiscolorationMap(bool flag)
{
    bGenerateDiscolorationMap=flag;
}

inline ColorDescriptor* RectangularTileMaker::GetTileDiscolorationMap()
{
    return (&tileColorDescriptorDiscoloration);
}

inline void RectangularTileMaker::DiscolorationMapTheresholdLower(double value)
{
    dDiscolorationThresholdLower=value;
}

inline void RectangularTileMaker::DiscolorationMapTheresholdUpper(double value)
{
    dDiscolorationThresholdUpper=value;
}

inline void RectangularTileMaker::DiscolorationMapTheresholdPercentReduction(double value)
{
    dDiscolorationThresholdPercentReduction=value;
}

inline void RectangularTileMaker::BlurFinalImage(bool flag)
{
    bBlurFinalImage=flag;
}

inline void RectangularTileMaker::SetOutputFileName(string fileName)
{
    outputFileName=fileName;
}

#endif /* RectangularTileMaker_hpp */
